/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

import java.util.*;
import java.io.*;

import minijava.node.*;
import minijava.lexer.*;
import minijava.parser.*;

class CommandLineProcessor {

    String slash, pathsep, java, kenya, gj , compiler , filename , packagename, code, temp;   
    Vector funcs;
    Vector classes;
    boolean tree;
    boolean compile, savejava;
    File file;

    CommandLineProcessor( String f, boolean c, boolean s , Properties prop ) {

	filename = f;
	compile = c;
	savejava = s;
		  
	slash = System.getProperty("file.separator");
	pathsep = System.getProperty("path.separator");
	java = prop.getProperty("java.location");
	kenya = prop.getProperty("kenya.location");	
	gj = prop.getProperty("gj.location");
	compiler = prop.getProperty("compiler");
	temp = prop.getProperty("working.dir");

	if ( filename.indexOf(".") >= 0 ) {
	    packagename = filename.substring( 0 , filename.indexOf(".") ); 
	} else {
	    packagename = filename;
	}

	if ( packagename.indexOf(slash) >= 0 ) {
	    packagename = packagename.substring( packagename.lastIndexOf(slash) + 1 , packagename.length() );
	}
    }
    
    void process() {
	
	try {

	    Lexer lexer = new Lexer ( new PushbackReader ( new FileReader( filename )));
	    Parser parser = new Parser( lexer );
	    Start ast = parser.parse();

	    funcs = new Vector();
	    classes = new Vector();
	    tree = true;


	    // Typecheck

	    ClassFinder cf = new ClassFinder();

	    ast.apply(cf);

	    TypeChecker tc = new TypeChecker((KenyaUI)null, cf.getClasses() , cf.getFunctions() );
	    tc.functions = cf.getFunctions();
	    tc.setNoWindows();

	    ast.apply(tc);
	    

	    // if typechecked ok ...

	    if ( tc.genCode ) {
		
		if ( savejava || compile ) {
		    
		    //make a directory
		    
		    File dir = new File(packagename);
		    if ( ! dir.exists() ) {
			dir.mkdir();
		    } 

		    //and create a file
		    
		    file = new File(packagename + slash + "Program.java");
	    
		    if ( ! file.exists() ) {
			file.createNewFile();
		    }
		}

		//translate kenya to java

		Translator t = new Translator(tc);		

		ast.apply(t);
		
		tree = false;
		
		//process functions

		for (Enumeration e = funcs.elements() ; e.hasMoreElements() ;) {
		    t.addLine("\n");
		    Node n = (Node)e.nextElement();
		    n.apply(t);
		}
		
		t.addLine("\n}\n");

		//process extra classes (structures)
		
		for (Enumeration e = classes.elements() ; e.hasMoreElements() ; ) {
		    Node n = (Node)e.nextElement();
		    n.apply(t);
		    t.addLine("\n");
		}
		
		Vector v = t.getCode();
		Iterator i = t.getImports();
		

		code = "package " + packagename + ";\n\n";
		
		if ( i.hasNext() ) {
		    
		    if ( ! compiler.equals("gj") ) {
			System.err.println("Warning: Your compiler (" + compiler + ") does not support parameterised types.");
		    }

		    while ( i.hasNext() ) {
			code = code.concat( "import gj.util." + ((String)i.next()).trim() + ";\n" );
		    }
		    
		    System.out.println("");
		}

		if ( t.usingGraphics )
		    code = code.concat( "import kenya.GraphicsWindow;\n\n" );
		
		if ( t.getReads() ) {
		    code = code.concat( "import kenya.io.InputReader;\n\n" );
		}
	
		code = code.concat("public class Program {\n");
		
	        Iterator constit  = t.getConstants();
		
		while ( constit.hasNext() ) {
		    Translator forconsts = new Translator();
		    ((Node)constit.next()).apply( forconsts );

		    for (Enumeration e = forconsts.getCode().elements() ; e.hasMoreElements() ; ) {
			CodeLine c = (CodeLine)e.nextElement();
			code = code.concat( c.code );
		    }
		}	        
		
		for (Enumeration e = v.elements() ; e.hasMoreElements() ; ) {
		    
		    CodeLine c = (CodeLine)e.nextElement();
		    
		    if ( c.newline ) {
			code = code.concat("\n");
			
			if ( ! e.hasMoreElements() ) 
			    break;
			
			CodeLine next = (CodeLine)e.nextElement();
			
			for ( int j = 0 ; j < next.indent ; j++ ) 
			    code = code.concat("\t");
			
			if ( next.code.equals("graphics") ) {
			    
			    if ( t.usingGraphics ) {
				code=code.concat("GraphicsWindow gw = new GraphicsWindow();\n\n");
				for ( int j = 0 ; j < next.indent ; j++ ) 
				    code = code.concat("\t");
			    }
			    
			} else {
			    
			    code = code.concat( next.code );
			}
		    }
			
		    code = code.concat( c.code );
		}

		Process comp = null;

		if ( savejava || compile ) {
		    
		    FileWriter outfile = new FileWriter( file );
		    outfile.write( code );
		    outfile.close();
		    
		    if ( compile &&  compiler.equals("gj") ) {

			String cmd = java + " -cp " + gj + slash + "classes" + pathsep + kenya + " gjc.Main -d " + temp + " " + temp + slash + packagename + slash + "Program.java";
			comp = Runtime.getRuntime().exec( cmd );

			if ( comp == null ) {
			    System.err.println("Unable to spawn external process. Error executing command : " + cmd );
			    System.exit(-1);
			}	

			comp.waitFor();
		    }

		    if ( compile && compiler.equals("javac") ) {

			String cmd = "javac -classpath " +  kenya + " -d " + temp + " " + temp + slash + packagename + slash + "Program.java";
			comp = Runtime.getRuntime().exec( cmd );	

			if ( comp == null ) {
			    System.err.println("Unable to spawn external process. Error executing command : " + cmd );
			    System.exit(-1);
			}

			comp.waitFor();
		    }		    

		} else {
		    System.out.println(code); 
		}

	    } else { System.exit(-1); }
	}
	
	catch ( minijava.parser.ParserException e ) {
	    System.err.println( e.getMessage() );
	    System.exit(-1);
	} 
	catch ( minijava.lexer.LexerException e ) {
	    System.err.println( e.getMessage() );
	    System.exit(-1);
	}
	catch ( FileNotFoundException fnf ) {
	    System.err.println( "File not found." );
	    System.exit(-1);
	}
	catch ( InterruptedException ie ) {
	    System.err.println( "Compilation did not complete properly." );
	    System.exit(-1);
	} 
	catch ( IOException e ) {
	    System.err.println( "An I/O error occurred while reading or writing a file.");
	    System.exit(-1);
	}
    }
}

